<?php
/*
Plugin Name: Exchange JPY
Plugin URI: 
Description: 外貨を円貨に換算して表示する
Version: 1.0.0
Author:REI MORI
Author URI: https://musyozoku.com
License: GPL2
*/
?>
<?php
/*  Copyright 2021 REI MORI (email : mori@musyozoku.com)
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
     published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
?>
<?php
defined( 'ABSPATH' ) or die( 'Nope, not accessing this' );
$Excange_JPY = Excange_JPY::get_instance();
//アクティベートとディアクティベート処理
register_activation_hook( __FILE__, array( $Excange_JPY, 'activate' ) );
register_deactivation_hook( __FILE__, array( $Excange_JPY, 'deactivate' ) );


class Excange_JPY {
	
	static $instance;

	public static function get_instance() {
        if( !isset( self::$instance ) ) {
        	$class = __CLASS__;
			self::$instance = new $class();
    	}
		return self::$instance;
	}
	
	// アクティベート
	public function activate() {
      $this->add_user_setting();
 	}
	
	// ディアクティベート
	public function deactivate() {
      $this->remove_user_setting();
 	}
 	
 	// アクティベートで呼ばれる処理
 	private function add_user_setting() {
	 	$exchange_setting = array(
		 	'exchange_api' => '',
		 	'exchange_decimal_point' => 0
	 	);
	 	update_option('exchange_setting', $exchange_setting);
	 	
	 	//cron登録
	 	if ( !wp_next_scheduled( 'delete-jpy-rate' ) ) {
		 	 wp_schedule_event( strtotime(date("Ymd 15:00:00")), 'daily', 'delete-jpy-rate' );
	 	}
 	}
 	
 	// 24時間で為替レートのデータを削除する
 	function delete_exchange_rate_dayly() {
	 	delete_option('exchange_rate');
	}
 	
 	// ディアクティベートで呼ばれる処理
 	private function remove_user_setting() {
      delete_option('exchange_setting');
      delete_option('exchange_rate');
      wp_clear_scheduled_hook('delete-jpy-rate');
 	}
	
	// コンストラクタ
	public function __construct() {
		add_action( 'admin_menu', array($this, 'add_plugin_admin_menu') ); //管理画面のメニュー
		add_shortcode("JPYexchange", array($this, 'fnc_jpyexchange')); //ショートコード
		add_action( 'delete-jpy-rate', array($this, 'delete_exchange_rate_dayly')); //毎日為替レートのデータを消す
	}
	
	// 2つ目、アクションフックで呼ばれる関数
	public function add_plugin_admin_menu() {
	     add_options_page(
	          '円貨換算の設定', // page_title（オプションページのHTMLのタイトル）
	          '円貨換算の設定', // menu_title（メニューで表示されるタイトル）
	          'administrator', // capability
	          'mo42-exchange-jpy', // menu_slug（URLのスラッグ）
	          array($this,'display_exchangeJPY_admin_page') // function
	     );
	     register_setting(
	          'mo42-exchange-jpy-group', // option_group
	          'exchange_setting', // option_name
	           array($this, 'exchange_api_validation') // sanitize_callback
	     );
	}
	
	// 設定項目のバリデーション
	public function exchange_api_validation($input) {
		if(!ctype_alnum($input['exchange_api'])) {
			add_settings_error(
		       'exchange_setting[exchange_api]',
		       'exchange_api-validation_error',
		       __( 'API Access Keyが不正です', 'APIキー' ),
		       'error'
	        );
		}
		if(!ctype_digit($input['exchange_decimal_point'])) {
			add_settings_error(
		       'exchange_setting[exchange_decimal_point]',
		       'exchange_decimal_point-validation_error',
		       __( '小数部桁数が不正です', '小数部桁数' ),
		       'error'
	        );
		}
		return $input;		
	}
	 
	// 3つ目、設定画面用のHTML
	public function display_exchangeJPY_admin_page() {
		 include_once( 'views/options.php' );
	}
	// END API設定画面
	
	// BEGIN ショートコード
	public function fnc_jpyexchange($atts) {
		$currency = $atts['currency'];
		$amount = $atts['amount'];
		if(empty($currency) || empty($amount)) return;
		
		$base_url = "http://apilayer.net/api/live"; //ベースのURL
		$setting = get_option("exchange_setting");
		$api_key = $setting['exchange_api']; //API Key
		$decimal_point = $setting['exchange_decimal_point']; //小数部桁数
		$currency_list = "JPY";
		$source = $currency; //ベースの通貨(無料プランの時はUSDベースしか取得できない。有料プランの場合は取得したい通貨コードを設定)
		//$currency_list = "GBP,ILS,INR,IDR,AUD,CAD,SGD,CHF,SEK,THB,CZK,CLP,DKK,TRY,NZD,NOK,PKR,HUF,PHP,BRL,PLN,MYR,MXN,EUR,RUB,CNY,ZAR,TWD,JPY,USD,KRW,HKD"; //通貨リスト(有料プランのみ有効)
		$requeat_url = $base_url . "?access_key=" . $api_key . "&currencies=" . $currency_list . "&source=" . $source . "&format=1";
		
		// DBに今日の為替があるかチェック
		$today_rate = get_option("exchange_rate");
		//$acquisition_date = $today_rate["date"];
		$today = time();
		
		if(!$today_rate || 
			($today - (int)$today_rate["date"]) > (60 * 60 * 24) ||
			!array_key_exists($currency, $today_rate)) {
			// 今日取得したデータがない、またはまったくデータがない場合、今日取得レートの中に対象通貨が含まれていない場合は新しく取得してDBに登録する
			$request = wp_remote_get($requeat_url);
			
			if ( is_wp_error( $request ) ) {
		        return false;
		    }
		    
		    $body = wp_remote_retrieve_body( $request );
		    $data = json_decode( $body, true );
		    
		    if (!$data["success"]) return; //無料プランでUSD以外を取得しようとした場合はなにも返さない
		    
		    $rate = $data["quotes"];
		    
			// DBに為替レートを追加する
			$exchange_rate = array(
				"date" => $data["timestamp"]
			);
			
			foreach ($rate as $key => $value) {
				// 取得したレートを登録
				$exchange_rate += array(str_replace('JPY', '', $key) =>$value);
			}
			
			if ($today_rate) {
				foreach ($today_rate as $key => $value) {
					// 以前に取得していた通貨レートを追加
					if ($key != 'date') {
						$exchange_rate += array($key => $value);
					}
				}
			}
			
			update_option("exchange_rate", $exchange_rate);
		}	
		
		// DBからレートを取得
		$today_rate = get_option("exchange_rate");
	    
	    if(!array_key_exists($currency, $today_rate)) return;
	    
	    $result = $today_rate[$currency] * $amount;
		return number_format($result, (int)$decimal_point);
	
	}
	// END ショートコード
}

?>